@@ -85,6 +85,7 @@
 #define PCI_CHIP_RV250_Ld 0x4C64
 #define PCI_CHIP_RV250_Lf 0x4C66
 #define PCI_CHIP_RV250_Lg 0x4C67
+#define PCI_CHIP_RV280_4C6E 0x4C6E
 #define PCI_CHIP_RAGE128MF 0x4D46
 #define PCI_CHIP_RAGE128ML 0x4D4C
 #define PCI_CHIP_R300_ND 0x4E44
@@ -451,6 +452,20 @@
 #define PCI_CHIP_RS780_9614 0x9614
 #define PCI_CHIP_RS780_9615 0x9615
 #define PCI_CHIP_RS780_9616 0x9616
+#define PCI_CHIP_SUMO_9640 0x9640
+#define PCI_CHIP_SUMO_9641 0x9641
+#define PCI_CHIP_SUMO2_9642 0x9642
+#define PCI_CHIP_SUMO2_9643 0x9643
+#define PCI_CHIP_SUMO2_9644 0x9644
+#define PCI_CHIP_SUMO2_9645 0x9645
+#define PCI_CHIP_SUMO_9647 0x9647
+#define PCI_CHIP_SUMO_9648 0x9648
+#define PCI_CHIP_SUMO_9649 0x9649
+#define PCI_CHIP_SUMO_964A 0x964A
+#define PCI_CHIP_SUMO_964B 0x964B
+#define PCI_CHIP_SUMO_964C 0x964C
+#define PCI_CHIP_SUMO_964E 0x964E
+#define PCI_CHIP_SUMO_964F 0x964F
 #define PCI_CHIP_RS880_9710 0x9710
 #define PCI_CHIP_RS880_9711 0x9711
 #define PCI_CHIP_RS880_9712 0x9712
@@ -463,6 +478,9 @@
 #define PCI_CHIP_PALM_9805 0x9805
 #define PCI_CHIP_PALM_9806 0x9806
 #define PCI_CHIP_PALM_9807 0x9807
+#define PCI_CHIP_PALM_9808 0x9808
+#define PCI_CHIP_PALM_9809 0x9809
+#define PCI_CHIP_PALM_980A 0x980A
 #define PCI_CHIP_CYPRESS_6880 0x6880
 #define PCI_CHIP_CYPRESS_6888 0x6888
 #define PCI_CHIP_CYPRESS_6889 0x6889
@@ -504,6 +522,7 @@
 #define PCI_CHIP_CEDAR_68F2 0x68F2
 #define PCI_CHIP_CEDAR_68F8 0x68F8
 #define PCI_CHIP_CEDAR_68F9 0x68F9
+#define PCI_CHIP_CEDAR_68FA 0x68FA
 #define PCI_CHIP_CEDAR_68FE 0x68FE
 #define PCI_CHIP_CAYMAN_6700 0x6700
 #define PCI_CHIP_CAYMAN_6701 0x6701
@@ -543,9 +562,22 @@
 #define PCI_CHIP_TURKS_6747 0x6747
 #define PCI_CHIP_TURKS_6748 0x6748
 #define PCI_CHIP_TURKS_6749 0x6749
+#define PCI_CHIP_TURKS_674A 0x674A
 #define PCI_CHIP_TURKS_6750 0x6750
+#define PCI_CHIP_TURKS_6751 0x6751
 #define PCI_CHIP_TURKS_6758 0x6758
 #define PCI_CHIP_TURKS_6759 0x6759
+#define PCI_CHIP_TURKS_675B 0x675B
+#define PCI_CHIP_TURKS_675D 0x675D
+#define PCI_CHIP_TURKS_675F 0x675F
+#define PCI_CHIP_TURKS_6840 0x6840
+#define PCI_CHIP_TURKS_6841 0x6841
+#define PCI_CHIP_TURKS_6842 0x6842
+#define PCI_CHIP_TURKS_6843 0x6843
+#define PCI_CHIP_TURKS_6849 0x6849
+#define PCI_CHIP_TURKS_6850 0x6850
+#define PCI_CHIP_TURKS_6858 0x6858
+#define PCI_CHIP_TURKS_6859 0x6859
 #define PCI_CHIP_CAICOS_6760 0x6760
 #define PCI_CHIP_CAICOS_6761 0x6761
 #define PCI_CHIP_CAICOS_6762 0x6762
@@ -556,4 +588,77 @@
 #define PCI_CHIP_CAICOS_6767 0x6767
 #define PCI_CHIP_CAICOS_6768 0x6768
 #define PCI_CHIP_CAICOS_6770 0x6770
+#define PCI_CHIP_CAICOS_6771 0x6771
+#define PCI_CHIP_CAICOS_6772 0x6772
+#define PCI_CHIP_CAICOS_6778 0x6778
 #define PCI_CHIP_CAICOS_6779 0x6779
+#define PCI_CHIP_CAICOS_677B 0x677B
+#define PCI_CHIP_ARUBA_9900 0x9900
+#define PCI_CHIP_ARUBA_9901 0x9901
+#define PCI_CHIP_ARUBA_9903 0x9903
+#define PCI_CHIP_ARUBA_9904 0x9904
+#define PCI_CHIP_ARUBA_9905 0x9905
+#define PCI_CHIP_ARUBA_9906 0x9906
+#define PCI_CHIP_ARUBA_9907 0x9907
+#define PCI_CHIP_ARUBA_9908 0x9908
+#define PCI_CHIP_ARUBA_9909 0x9909
+#define PCI_CHIP_ARUBA_990A 0x990A
+#define PCI_CHIP_ARUBA_990F 0x990F
+#define PCI_CHIP_ARUBA_9910 0x9910
+#define PCI_CHIP_ARUBA_9913 0x9913
+#define PCI_CHIP_ARUBA_9917 0x9917
+#define PCI_CHIP_ARUBA_9918 0x9918
+#define PCI_CHIP_ARUBA_9919 0x9919
+#define PCI_CHIP_ARUBA_9990 0x9990
+#define PCI_CHIP_ARUBA_9991 0x9991
+#define PCI_CHIP_ARUBA_9992 0x9992
+#define PCI_CHIP_ARUBA_9993 0x9993
+#define PCI_CHIP_ARUBA_9994 0x9994
+#define PCI_CHIP_ARUBA_99A0 0x99A0
+#define PCI_CHIP_ARUBA_99A2 0x99A2
+#define PCI_CHIP_ARUBA_99A4 0x99A4
+#define PCI_CHIP_TAHITI_6780 0x6780
+#define PCI_CHIP_TAHITI_6784 0x6784
+#define PCI_CHIP_TAHITI_6788 0x6788
+#define PCI_CHIP_TAHITI_678A 0x678A
+#define PCI_CHIP_TAHITI_6790 0x6790
+#define PCI_CHIP_TAHITI_6791 0x6791
+#define PCI_CHIP_TAHITI_6792 0x6792
+#define PCI_CHIP_TAHITI_6798 0x6798
+#define PCI_CHIP_TAHITI_6799 0x6799
+#define PCI_CHIP_TAHITI_679A 0x679A
+#define PCI_CHIP_TAHITI_679E 0x679E
+#define PCI_CHIP_TAHITI_679F 0x679F
+#define PCI_CHIP_PITCAIRN_6800 0x6800
+#define PCI_CHIP_PITCAIRN_6801 0x6801
+#define PCI_CHIP_PITCAIRN_6802 0x6802
+#define PCI_CHIP_PITCAIRN_6806 0x6806
+#define PCI_CHIP_PITCAIRN_6808 0x6808
+#define PCI_CHIP_PITCAIRN_6809 0x6809
+#define PCI_CHIP_PITCAIRN_6810 0x6810
+#define PCI_CHIP_PITCAIRN_6811 0x6811
+#define PCI_CHIP_PITCAIRN_6816 0x6816
+#define PCI_CHIP_PITCAIRN_6817 0x6817
+#define PCI_CHIP_PITCAIRN_6818 0x6818
+#define PCI_CHIP_PITCAIRN_6819 0x6819
+#define PCI_CHIP_PITCAIRN_684C 0x684C
+#define PCI_CHIP_VERDE_6820 0x6820
+#define PCI_CHIP_VERDE_6821 0x6821
+#define PCI_CHIP_VERDE_6823 0x6823
+#define PCI_CHIP_VERDE_6824 0x6824
+#define PCI_CHIP_VERDE_6825 0x6825
+#define PCI_CHIP_VERDE_6826 0x6826
+#define PCI_CHIP_VERDE_6827 0x6827
+#define PCI_CHIP_VERDE_6828 0x6828
+#define PCI_CHIP_VERDE_6829 0x6829
+#define PCI_CHIP_VERDE_682B 0x682B
+#define PCI_CHIP_VERDE_682D 0x682D
+#define PCI_CHIP_VERDE_682F 0x682F
+#define PCI_CHIP_VERDE_6830 0x6830
+#define PCI_CHIP_VERDE_6831 0x6831
+#define PCI_CHIP_VERDE_6837 0x6837
+#define PCI_CHIP_VERDE_6838 0x6838
+#define PCI_CHIP_VERDE_6839 0x6839
+#define PCI_CHIP_VERDE_683B 0x683B
+#define PCI_CHIP_VERDE_683D 0x683D
+#define PCI_CHIP_VERDE_683F 0x683F
